from typing import TYPE_CHECKING
from typing import Any

from django.conf import settings
from django.core.mail import get_connection
from django.core.mail.backends.base import BaseEmailBackend
from django_q.tasks import async_task

from . import utils

if TYPE_CHECKING:
    from django.core.mail import EmailMessage

    from .typing import EmailMessageData

Q2_EMAIL_BACKEND = getattr(
    settings, "Q2_EMAIL_BACKEND", "django.core.mail.backends.smtp.EmailBackend"
)


class Q2EmailBackend(BaseEmailBackend):
    def __init__(
        self,
        fail_silently: bool = False,
        **kwargs: Any,  # NOQA: ANN401
    ) -> None:
        super().__init__(fail_silently)
        self.init_kwargs = kwargs

    def send_messages(self, email_messages: list["EmailMessage"]) -> int:
        num_sent = 0
        for email_message in email_messages:
            serialized_email_message = utils.to_dict(email_message)
            async_task(self.send_message, serialized_email_message)
            num_sent += 1
        return num_sent

    def send_message(self, serialized_email_message: "EmailMessageData") -> None:
        email_message = utils.from_dict(serialized_email_message)
        email_message.connection = get_connection(
            backend=Q2_EMAIL_BACKEND, **self.init_kwargs
        )
        email_message.send()



from __future__ import annotations

from typing import TYPE_CHECKING
from typing import TypedDict

if TYPE_CHECKING:
    from email.mime.base import MIMEBase

    class RequiredEmailMessageData(TypedDict):
        cc: list[str] | None
        subject: str
        body: str
        from_email: str | None
        to: list[str] | None
        bcc: list[str] | None
        attachments: list[MIMEBase | tuple[str, str]] | None
        headers: dict[str, str] | None
        reply_to: list[str] | None

    class OptionalEmailMessageData(TypedDict, total=False):
        alternatives: list[tuple[str, str]] | None

    class EmailMessageData(RequiredEmailMessageData, OptionalEmailMessageData): ...



from typing import TYPE_CHECKING

from django.core.mail import EmailMessage
from django.core.mail import EmailMultiAlternatives

if TYPE_CHECKING:
    from .typing import EmailMessageData


def to_dict(email_message: EmailMessage) -> dict[str, str]:
    email_message_data = {
        "cc": email_message.cc,
        "subject": email_message.subject,
        "body": email_message.body,
        "from_email": email_message.from_email,
        "to": email_message.to,
        "bcc": email_message.bcc,
        "attachments": email_message.attachments,
        "headers": email_message.extra_headers,
        "reply_to": email_message.reply_to,
    }
    if isinstance(email_message, EmailMultiAlternatives):
        email_message_data["alternatives"] = email_message.alternatives
    return email_message_data


def from_dict(email_message_data: "EmailMessageData") -> EmailMessage:
    kwargs = dict(email_message_data)
    if alternatives := kwargs.pop("alternatives", None):
        return EmailMultiAlternatives(alternatives=alternatives, **kwargs)
    return EmailMessage(**kwargs)



from django.apps import AppConfig


class Q2EmailBackendConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_q2_email_backend"
    label = "q2_email_backend"
